# from aws_cdk import (
#     aws_ssm as ssm,
#     aws_secretsmanager as sm,
#     core
# )

# # Outside contexts/stacks
# # app.node.try_get_context('CONTEXT_KEY')

# class dummy(core.Construct):
#     def __init__(self, scope: core.Construct, id: str):
#         super().__init__(scope, id)

#         # From SSM
#         SSMVALUE = ssm.StringParameter.value_from_lookup(self, parameter_name='/SSM/PARAM/PATH')

#         # or
#         account = '424242'
#         region = 'eu-west-1'
#         ssmp = "ssm:account={}:parameterName=/aviv/cfn/custom/layer:region={}".format(account, region)
#         self.node.try_get_context(ssmp)
#         app.node.try_get_context(ssmp)

#         # do not use unless you WANT to create a dependency between stacks
#         core.Fn.import_value('CFN_STACK_EXPORT_VALUE')



import os
import logging
from aws_cdk import (
    aws_iam as iam,
    aws_lambda,
    aws_ssm as ssm,
    aws_cloudformation as cfn,
    core
)
from .cdk_lambda import CDKLambda


class IAMIdpSAML(CDKLambda):
    _idp: cfn.CfnCustomResource = None

    def __init__(self,  scope: core.Construct, id: str, idp_name: str, idp_url: str, *, cfn_resources_path: str=None, debug=False):
        """Create an IAM SAML Identity Provider

        Args:
            scope (core.Construct): [description]
            id (str): [description]
            idp_name (str): IAM Idp name
            idp_url (str): Your SAML Identity provider URL
        """
        rdir = os.path.dirname(os.path.dirname(__file__))
        lambda_attrs=dict(
                code=aws_lambda.InlineCode(CDKLambda._code_inline(rdir + '/lambdas/iam_idp/saml.py')),
                handler='index.handler',
                timeout=core.Duration.seconds(20),
                runtime=aws_lambda.Runtime.PYTHON_3_7
        )
        # TODO: release layer
        # pip install -r requirements.txt -t .
        if not cfn_resources_path:
            cfn_resources_path=rdir + '/build/artifacts-cfn_resources.zip'
        layer_attrs=dict(
            description='cfn_resources layer for idp',
            code=aws_lambda.AssetCode(cfn_resources_path)
        )
        super().__init__(scope, id, lambda_attrs=lambda_attrs, layer_attrs=layer_attrs, remote_account_grant=False)

        # Add required policies for the lambda to create an IAM idp
        self._lambda.add_to_role_policy(
            iam.PolicyStatement(
                actions=['iam:CreateSAMLProvider', 'iam:UpdateSAMLProvider', 'iam:DeleteSAMLProvider'],
                effect=iam.Effect.ALLOW,
                resources=['*']
            )
        )

        self._idp = cfn.CustomResource(
            self, "identityProvider",
            resource_type='Custom::SAMLProvider',
            provider=cfn.CustomResourceProvider.lambda_(self._lambda),
            properties=dict(
                Name=idp_name,
                URL=idp_url
            )
        )
        self.response = self._idp.get_att("Response").to_string()

        # Export
        ssm_name = '/' + id.replace('-', '/')
        ssm.StringParameter(self, 'ssm', string_value=self._idp.ref, parameter_name=ssm_name)
        core.CfnOutput(self, 'IAMIdpSAMLArn', value=self._idp.ref)

    @property
    def arn(self):
        return self._idp.ref

    @property
    def idp(self):
        return self._idp



import os
import yaml
import logging
from aws_cdk import (
    aws_codebuild as cb,
    aws_codepipeline as cp,
    aws_codepipeline_actions as cpa,
    aws_codecommit as cc,
    aws_codestarconnections as csc,
    pipelines as cdkpipeline,
    aws_s3 as s3,
    core
)

CONN = os.environ.get('arn:aws:codestar-connections:eu-west-1:605901617242:connection/675a0f98-dd7c-4d72-86c8-57378c507e5d')



class Pipelines(core.Construct):
    artifacts = {
        'sources': [],
        'builds': [],
        'build_extras': [],
        'deploy': []
    }
    actions = {
        'sources': [],
        'builds': [],
        'build_extras': [],
        'deploy': []
    }

    def __init__(self, scope, id, github_config: dict, project_config: dict):
        super().__init__(scope, id)
        self.bucket = s3.Bucket(self, 'bucket',
            removal_policy=core.RemovalPolicy.RETAIN,
            encryption=s3.BucketEncryption.KMS_MANAGED,
            versioned=True
        )
        self.pipe = cp.Pipeline(self, 'pipe', cross_account_keys=True, pipeline_name=id + 'Pipe')

        self._source(**github_config)
        self._project(**project_config)
        self._build(self.artifacts['sources'][0], self.artifacts['build_extras'])
        # self._deploy()

    def _project(self, **project_config):
        if 'build_spec' in project_config:
            project_config['build_spec'] = Pipelines.load_buildspec(project_config['build_spec'])
        self.project = cb.PipelineProject(
            self, "project",
            project_name="{}".format(self.node.id),
            environment=cb.LinuxBuildImage.STANDARD_4_0,
            cache=cb.Cache.bucket(bucket=self.bucket, prefix='codebuild-cache'),
            **project_config
        )

    def _source(self, **github_config):
        artifact = cp.Artifact()

        conn =csc.CfnConnection(
            self, 'ghconn',
            connection_name='{}'.format(github_config['owner']),
            host_arn=CONN,
            #'{}/{}'.format(github_config['owner'], github_config['repo']),
            provider_type='GitHub'
        )
        # cc.Repository(
        #     self, 'ghrepo',
        #     repository_name=github_config['repo']
        # )
        # cpa.CodeCommitSourceAction(
        #     self, 'ccaction',
        #     output=artifact,
        #     repository=
        # )

        checkout = cpa.GitHubSourceAction(
            action_name="{}@{}".format(github_config['repo'], github_config['branch']),
            output=artifact,
            trigger=cpa.GitHubTrigger.POLL,
            **github_config
        )
        self.artifacts['sources'].append(artifact)
        self.actions['sources'].append(checkout)
        self.pipe.add_stage(stage_name='Source@{}'.format(github_config['repo']), actions=[checkout])

    def _build(self, input, extra_inputs=[]):
        artifact = cp.Artifact()
        build = cpa.CodeBuildAction(
            outputs=[artifact],
            type=cpa.CodeBuildActionType.BUILD,
            action_name="Build",
            input=input,
            extra_inputs=extra_inputs,
            project=self.project
        )
        self.artifacts['builds'].append(artifact)
        self.actions['builds'].append(build)

        self.pipe.add_stage(
            stage_name="Build",
            actions=self.actions['builds']
        )

    def _deploy(self, **deploy_config):
        deploy = cpa.CloudFormationCreateUpdateStackAction(
            action_name='Deploy',
            stack_name='iam-idp',
            admin_permissions=True,
            template_path=self.artifacts['builds'][0].at_path("cdk.out/iam-idp.template.json"),
            **deploy_config
        )
        self.actions['deploy'].append(deploy)

    @staticmethod
    def env(environment_variables: dict):
        envs = dict()
        for env, value in environment_variables.items():
            envs[env] = cb.BuildEnvironmentVariable(value=value)
        return envs
    
    @staticmethod
    def load_buildspec(specfile):
        with open(specfile, encoding="utf8") as fp:
            bsfile = fp.read()
            bs = yaml.safe_load(bsfile)
            return cb.BuildSpec.from_object(value=bs)



import os
from aws_cdk import (
    aws_ssm as ssm,
    aws_lambda,
    core
)

class CDKLambda(core.Construct):
    _layer = None
    _lambda = None
    _assets: dict = None

    def __init__(self,  scope: core.Construct, id: str, *, lambda_attrs: aws_lambda.FunctionProps=None, layer_attrs=None, remote_account_grant=False, use_layer=True, cfn_outputs=False):
        """Provides a CDK Construct for AWS Lambda and Layers

        Args:
            scope (core.Construct): [description]
            id (str): [description]
            lambda_attrs (aws_lambda.FunctionProps, optional): [description]. Defaults to None.
            layer_attrs (aws_lambda.LayerVersionProps, optional): [description]. Defaults to None.
            remote_account_grant (bool, optional): [description]. Defaults to False.
            use_layer (bool, optional): [description]. Defaults to True.
        """
        super().__init__(scope, id)

        if layer_attrs:
            self._layer = aws_lambda.LayerVersion(
                self, "layer",
                **layer_attrs
            )
            ssm_name = "/{}/layer".format(id.replace('-', '/'))
            ssm.StringParameter(self, 'Layer', string_value=self._layer.layer_version_arn, parameter_name=ssm_name)
            if remote_account_grant:
                self._layer.add_permission('remote-account-grant', account_id='*')

        if lambda_attrs:
            self._lambda = aws_lambda.Function(
                self, "lambda", **lambda_attrs
            )
            if self._layer and use_layer:
                self._lambda.add_layers(self._layer)
        if cfn_outputs:
            core.CfnOutput(self, 'LambdaArn', value=self._lambda.function_arn)
            if self._layer:
                core.CfnOutput(self, 'LayerArn', value=self._layer.from_layer_version_arn)


    @staticmethod
    def _code_inline(filepath):
        """Streamline file into a string for a simple AWS Lambda

        Args:
            filepath ([type]): [description]

        Returns:
            [type]: [description]
        """
        with open(filepath, encoding="utf8") as fp:
            code = fp.read()
        return code



from aws_cdk import (
    aws_iam as iam,
    aws_secretsmanager as asm,
    core
)


class SecurityCredentials(core.Construct):
    user: iam.IUser

    def __init__(self, scope: core.Construct, id: str, *, user: iam.IUser=None) -> None:
        super().__init__(scope, id)
        self.user = user

    def console_password(self, secret_name: str, template: str = None, key: str = None):
        self.secret = asm.Secret(
            self,
            id,
            generate_secret_string=asm.SecretStringGenerator(
                secret_string_template=template,
                generate_string_key=key,
                password_length=24,
                exclude_characters='"@/\$'
            ),
            secret_name='{}{}'.format(secret_name, id)
        )
        return core.SecretValue(self.secret.secret_value.to_string())

    def access_key(self, user_name: str=None):
        if not user_name and self.user:
            user_name = self.user.user_name
        userkey = iam.CfnAccessKey(self, 'key', user_name=user_name)

        secret = asm.CfnSecret(
            self, 'userKey',
            name='iam/user/{}/accesskey'.format(user_name),
            secret_string='{{"AccessKeyId": "{}", "SecretAccessKey": "{}"}}'.format(
                userkey.ref,
                userkey.attr_secret_access_key
            )
        )
        return userkey.attr_secret_access_key

class Role(iam.Role):
    def __init__(self, scope: core.Construct, id: str, assumed_by=None, *, description=None, policies: list=None, external_ids=None, inline_policies=None,
        managed_policies=None, max_session_duration=None, path=None, permissions_boundary=None) -> None:
        """Same as a regular CDK AWS IAM Role except the Role name is defined by the CDK object 'id'.

        Args:
            scope (Construct): CDK Contruct/Stack
            id (str): Role id
            assumed_by: The IAM principal which can assume this role. You can later modify the assume role policy document by accessing it via the ``assumeRolePolicy`` property.
            description: A description of the role. It can be up to 1000 characters long. Default: - No description.
            policies: A list of AWS pre-defined IAM policies (lookup in arn:aws:iam::aws:policy/)
            external_ids: List of IDs that the role assumer needs to provide one of when assuming this role. If the configured and provided external IDs do not match, the AssumeRole operation will fail. Default: No external ID required
            inline_policies: A list of named policies to inline into this role. These policies will be created with the role, whereas those added by ``addToPolicy`` are added using a separate CloudFormation resource (allowing a way around circular dependencies that could otherwise be introduced). Default: - No policy is inlined in the Role resource.
            managed_policies: A list of managed policies associated with this role. You can add managed policies later using ``addManagedPolicy(ManagedPolicy.fromAwsManagedPolicyName(policyName))``. Default: - No managed policies.
            max_session_duration: The maximum session duration that you want to set for the specified role. This setting can have a value from 1 hour (3600sec) to 12 (43200sec) hours. Anyone who assumes the role from the AWS CLI or API can use the DurationSeconds API parameter or the duration-seconds CLI parameter to request a longer session. The MaxSessionDuration setting determines the maximum duration that can be requested using the DurationSeconds parameter. If users don't specify a value for the DurationSeconds parameter, their security credentials are valid for one hour by default. This applies when you use the AssumeRole* API operations or the assume-role* CLI operations but does not apply when you use those operations to create a console URL. Default: Duration.hours(1)
            path: The path associated with this role. For information about IAM paths, see Friendly Names and Paths in IAM User Guide. Default: /
            permissions_boundary: AWS supports permissions boundaries for IAM entities (users or roles). A permissions boundary is an advanced feature for using a managed policy to set the maximum permissions that an identity-based policy can grant to an IAM entity. An entity's permissions boundary allows it to perform only the actions that are allowed by both its identity-based policies and its permissions boundaries. Default: - No permissions boundary.
        """
        if policies:
            # Here the scope is the higher level construct/node, NOT self as the lookup is made before *this* construct calls its parent
            mpolicies = [
                iam.ManagedPolicy.from_managed_policy_arn(
                    # scope, '{}-{}-{}'.format(mpname, id, scope.node.id),
                    scope, '{}-{}'.format(mpname, id),
                    managed_policy_arn='arn:aws:iam::aws:policy/{}'.format(mpname)
                ) for mpname in policies
            ]
            if not managed_policies:
                managed_policies = list()
            managed_policies += mpolicies
        super().__init__(
            scope=scope, id=id, assumed_by=assumed_by, description=description, external_ids=external_ids, inline_policies=inline_policies,
            managed_policies=managed_policies, max_session_duration=max_session_duration, path=path, permissions_boundary=permissions_boundary, role_name=id
        )


class SAMLRole(Role):
    def __init__(self, scope: core.Construct, id: str, federated: str, *, description=None, external_ids=None, inline_policies=None,
        managed_policies=None, max_session_duration=None, path=None, permissions_boundary=None) -> None:
        """Same as CDK IAM Role above. The assumed_by is a FederatedPrincipal 'federated' (your IAM Idp arn).

        Args:
            federated: ARN of the AWS IAM Idp through which users assume this IAM Role
        """
        assumed_by=iam.FederatedPrincipal(
            federated=federated,
            conditions={
                'StringEquals': {'SAML:aud': 'https://signin.aws.amazon.com/saml'}
            },
            assume_role_action='sts:AssumeRoleWithSAML'
        )

        super().__init__(
            scope=scope, id=id, assumed_by=assumed_by, description=description, external_ids=external_ids, inline_policies=inline_policies,
            managed_policies=managed_policies, max_session_duration=max_session_duration, path=path, permissions_boundary=permissions_boundary
        )



import setuptools
import aviv_cdk

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="aviv-cdk", # Replace with your own username
    version=aviv_cdk.__version__,
    author="Jules Clement",
    author_email="jules.clement@aviv-group.com",
    description="Aviv CDK Python library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/aviv-group/aviv-cdk-python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
         "boto3>=1.14",
         "aws-cdk-core>=1.68",
         "aws-cdk-aws-iam",
         "aws-cdk-aws-s3",
         "aws-cdk-aws-lambda",
         "aws-cdk-aws-ssm",
         "aws-cdk-aws-secretsmanager"
   ],
    extras_require={
        "cicd": ["pyyaml", "aws-cdk-pipelines", "aws-cdk-aws-codebuild", "aws-cdk-aws-codepipeline", "aws-cdk-aws-codepipeline-actions"],
        "data": ["aws-cdk-glue", "aws-cdk-athena"]
    },
    python_requires='>=3.6',
)
